/* Reaction diffusion equation in one-dimensional space
   Logistig growth
   Uses two nodes */

#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

#define SIZE 50

#define DT   0.0001
#define DX   0.01

#define STEP 500000
#define INTV 5000

void initialize(int), process(int), go_ahead(int), finish();
double f(double);

double n[SIZE+2], nd[SIZE+2], c;

int main(int argc, char *argv[])
{
  int rank, size, namelen;
  char processor_name[MPI_MAX_PROCESSOR_NAME];

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  MPI_Get_processor_name(processor_name, &namelen);

  printf("Process %d of %d on %s\n", rank, size, processor_name);

  initialize(rank);
  process(rank);

  finish();
  
  MPI_Finalize();

  return 0;
}

void initialize(int rank)
{
  int i;

  c = 0.001*DT/DX/DX;

  for(i=1; i<= SIZE; i++)
    n[i] = 0.0;

  if( rank == 0 )
    n[1] = 0.01;

}

void process(int rank)
{
  int t;

  for(t=0; t<STEP; t++){

    if( t%INTV == 0 )
      if( rank == 0 )
		printf("Step = %d, %f\n", t, n[1]);
    
    go_ahead(rank);
  
  }

}

void go_ahead(int rank)
{
  MPI_Status mpi_stat;
  double n_2;
  int dest, source, tag;
  int i;
  
  if( rank == 0 ){
    
    dest = 1;
    tag = 7;
    MPI_Send(&n[SIZE], 1, MPI_DOUBLE, dest, tag, MPI_COMM_WORLD);
    
    source = 1;
    tag = 77;
    MPI_Recv(&n_2, 1, MPI_DOUBLE, source, tag, MPI_COMM_WORLD, &mpi_stat);
    
    n[0] = n[1];
    n[SIZE+1] = n_2;
    
  } else if( rank == 1) {
    
    source = 0;
    tag = 7;
    MPI_Recv(&n_2, 1, MPI_DOUBLE, source, tag, MPI_COMM_WORLD, &mpi_stat);
	
    dest = 0;
    tag = 77;
    MPI_Send(&n[1], 1, MPI_DOUBLE, dest, tag, MPI_COMM_WORLD);
   
	n[0] = n_2;
    n[SIZE+1] = n[SIZE];

  }
  
  for(i=1; i<=SIZE; i++)
    nd[i] = n[i] + c*(n[i-1] -2*n[i] + n[i+1]) + f(n[i])*DT;
  
  for(i=1; i<=SIZE; i++)
    n[i] = nd[i];
  
}

void finish()
{
}

double f(double x)
{
  double tmp;

  tmp = x*(1-x);

  return tmp;
}
